typedef union {
	int i;
	bpf_u_int32 h;
	u_char *e;
	char *s;
	struct stmt *stmt;
	struct arth *a;
	struct {
		struct qual q;
		struct block *b;
	} blk;
	struct block *rblk;
} YYSTYPE;
#define	DST	258
#define	SRC	259
#define	HOST	260
#define	GATEWAY	261
#define	NET	262
#define	MASK	263
#define	PORT	264
#define	LESS	265
#define	GREATER	266
#define	PROTO	267
#define	PROTOCHAIN	268
#define	BYTE	269
#define	ARP	270
#define	RARP	271
#define	IP	272
#define	TCP	273
#define	UDP	274
#define	ICMP	275
#define	IGMP	276
#define	IGRP	277
#define	PIM	278
#define	ATALK	279
#define	DECNET	280
#define	LAT	281
#define	SCA	282
#define	MOPRC	283
#define	MOPDL	284
#define	TK_BROADCAST	285
#define	TK_MULTICAST	286
#define	NUM	287
#define	INBOUND	288
#define	OUTBOUND	289
#define	LINK	290
#define	GEQ	291
#define	LEQ	292
#define	NEQ	293
#define	ID	294
#define	EID	295
#define	HID	296
#define	HID6	297
#define	LSH	298
#define	RSH	299
#define	LEN	300
#define	IPV6	301
#define	ICMPV6	302
#define	AH	303
#define	ESP	304
#define	OR	305
#define	AND	306
#define	UMINUS	307


extern YYSTYPE pcap_lval;
