/* nbase_config.h.  Generated automatically by configure.  */
#ifndef NBASE_CONFIG_H
#define NBASE_CONFIG_H

#define HAVE_USLEEP 1

#define HAVE_NANOSLEEP 1

/* #undef HAVE_STRUCT_ICMP */

/* #undef HAVE_IP_IP_SUM */

/* #undef inline */

#define STDC_HEADERS 1

#define HAVE_STRING_H 1

#define HAVE_NETDB_H 1

#define HAVE_GETOPT_H 1

#define HAVE_UNISTD_H 1

#define HAVE_STRINGS_H 1

/* #undef HAVE_BSTRING_H */

/* #undef WORDS_BIGENDIAN */

#define HAVE_MEMORY_H 1

/* both bzero() and memcpy() are used in the source */
#define HAVE_BZERO 1
#define HAVE_MEMCPY 1
#define HAVE_STRERROR 1

#define HAVE_SYS_PARAM_H 1

/* #undef HAVE_SYS_SOCKIO_H */

#define HAVE_SYS_SOCKET_H 1

#define HAVE_SYS_WAIT_H 1

/* #undef BSD_NETWORKING */

#define HAVE_STRCASESTR 1

#define HAVE_STRCASECMP 1

#define HAVE_STRNCASECMP 1

#define HAVE_GETTIMEOFDAY 1

#define HAVE_SLEEP 1

#define HAVE_SIGNAL 1

/* #undef HAVE_GETOPT_LONG */

/* #undef HAVE_NETINET_IN_SYSTEM_H */

/* #undef HAVE_SOCKADDR_SA_LEN */

#define HAVE_NETINET_IF_ETHER_H 1

#define HAVE_NETINET_IN_H 1

#define HAVE_SYS_TIME_H 1

/* #undef PWD_H */

#define HAVE_ARPA_INET_H 1

#define HAVE_SYS_RESOURCE_H 1

#define HAVE_RPC_TYPES_H 1

#define HAVE_SYS_STAT_H 1

/* #undef SPRINTF_RETURNS_STRING */

/* Integer widths */
#define SIZEOF_CHAR 1
#define SIZEOF_SHORT 2
#define SIZEOF_INT 4
#define SIZEOF_LONG 4

/* Define if you have the snprintf function.  */
#define HAVE_SNPRINTF 1

/* Define if you have the vasnprintf function.  */
/* #undef HAVE_VASNPRINTF */

/* Define if you have the vasprintf function.  */
#define HAVE_VASPRINTF 1

/* Define if you have the vfprintf function.  */
/* #undef HAVE_VFPRINTF */

/* Define if you have the vsnprintf function.  */
#define HAVE_VSNPRINTF 1

/* define if your compiler has __attribute__ */
#define HAVE___ATTRIBUTE__ 1

#define LINUX 1
/* #undef FREEBSD */
/* #undef OPENBSD */
/* #undef SOLARIS */
/* #undef SUNOS */
/* #undef BSDI */
/* #undef IRIX */
/* #undef NETBSD */

#endif /* NBASE_CONFIG_H */


